a {
  text-decoration: none;
}

.case {
  margin-bottom: 5.0625em;
}

.test-case-heading,
.input-heading,
.output-heading,
.expected-heading {
  font-family: monospace;
  text-transform: uppercase;
  border-bottom: 1px dashed;
  padding: 2px 0;
}

.test-case-heading {
  border-bottom: 3px solid;
}

.output,
.expected {
  padding: 1em;
}

.output {
  background-color: #e6e6e6;
}

.output--err {
  background-color: red;
  color: #fff;
}

.expected {
  background-color: #000;
  color: #fff;
}

.expected,
.expected-heading {
  display: none;
}

.expected--err,
.expected-heading--err {
  display: block;
}
